-- Mass-select people from the invite list to modify their statuses
-- 60

local core = CalendarExtras

local module = core:NewModule("Mass Select")

 --[[ -------- Locals ------- ]]--

local selectedByName = {}
local selectedByIndex = setmetatable({}, {__newindex = function(self, key, value) -- Generate name-table automatically from the index-table
	rawset(self, key, value)
	if value ~= nil then
		selectedByName[value] = true
	elseif key then
		local name = CalendarEventGetInvite(key)
		if name then
			selectedByName[name] = nil
		end
	end
end})
local lastSelectedIndex, lastSelectedName
local listIsDirty

local currentInviteList -- The currently open scrollframe (CalendarViewEventInviteListScrollFrame or CalendarCreateEventInviteListScrollFrame)

local highlightTexture

local underline

local sortersModule

 --[[ ----------------------- ]]--

function module:Select(button)
	local index
	if type(button) == "table" then
		index = button.inviteIndex
		local hlTexture = button:GetHighlightTexture()
		if not hlTexture:GetTexture() then hlTexture:SetTexture(highlightTexture) end
	else
		index = button
	end
	selectedByIndex[index] = CalendarEventGetInvite(index)
end

function module:RangeSelect(from, to)
	for i = from, to, from > to and -1 or 1 do
		self:Select(i)
	end
end

function module:Unselect(button)
	selectedByIndex[button.inviteIndex] = nil
end

function module:UnselectAll()
	wipe(selectedByIndex)
	wipe(selectedByName)
end

function module:OnClick(button)
	if button == "LeftButton" then
		if IsShiftKeyDown() and IsControlKeyDown() then -- Add range select
			module:RangeSelect(lastSelectedIndex, self.inviteIndex)
		elseif IsShiftKeyDown() then -- Range select
			module:UnselectAll()
			module:RangeSelect(lastSelectedIndex, self.inviteIndex)
		else
			if IsControlKeyDown() then -- Add to selection
				if selectedByIndex[self.inviteIndex] then
					module:Unselect(self)
					self:GetHighlightTexture():SetTexture() -- Clear the highlight texture temporarily so the button under the cursor unhighlights when unselected. Go go gadget hugeuglyhack #1148
				else
					module:Select(self)
				end
			else -- Single select
				module:UnselectAll()
				module:Select(self)
			end
			lastSelectedIndex, lastSelectedName = self.inviteIndex, CalendarEventGetInvite(self.inviteIndex)
			underline:Reposition()
		end
	elseif button == "RightButton" then
		if not selectedByIndex[self.inviteIndex] then -- Unselect everything if we didn't right-click something that was already selected
			module:UnselectAll()
		end
		module:Select(self)
		lastSelectedIndex, lastSelectedName = self.inviteIndex, CalendarEventGetInvite(self.inviteIndex)
		underline:Reposition()
	end
	module:UpdateInviteListButtons()
end

function module:OnEnter()
	underline.shouldShow = true -- This is mainly to hide it when the event frame is first shown... I think...
	underline:Reposition()
end

function module:OnLeave()
	if self.GetHighlightTexture then -- The ScrollFrame which we HookScript doesn't have this
		local texture = self:GetHighlightTexture()
		if not texture:GetTexture() then texture:SetTexture(highlightTexture) end
	end
	underline.shouldShow = nil
	underline:Hide()
end

-- Rebuild the selectedByIndex table from selectedByName when the indices change, i.e. when the list is re-sorted or the list otherwise changes
function module:RebuildSelectedIndices()
	if not next(selectedByName) then return end
	wipe(selectedByIndex)
	local i, name = 1
	while true do
		name = CalendarEventGetInvite(i)
		if not name then break end
		if selectedByName[name] then
			selectedByIndex[i] = name
		end
		if name == lastSelectedName then
			lastSelectedIndex = i
			underline:Reposition()
		end
		i = i + 1
	end
	self:UpdateInviteListButtons() -- This is kinda stupid, but the list update hooks fire before the "CALENDAR_UPDATE_INVITE_LIST" event reaches this module >_>
end

-- Update the highlights on the visible buttons according to selection
function module:UpdateInviteListButtons(_, frame)
	if listIsDirty then
		listIsDirty = nil
		self:RebuildSelectedIndices()
	end
	if frame then currentInviteList = frame end
	for _, button in ipairs(currentInviteList.buttons) do
		if button.inviteIndex and selectedByIndex[button.inviteIndex] then
			button:LockHighlight()
		else
			button:UnlockHighlight()
		end
	end
	underline:Reposition()
end

-- Let other modules etc run functions on the selected stuff
function module:RunOnSelected(func, useName, value)
	for index in pairs(useName and selectedByName or selectedByIndex) do
		func(index, value)
	end
	self:RebuildSelectedIndices()
end

function module:OnInitialize()
	if core.modules.Sorters then
		sortersModule = core.modules.Sorters
	end

	-- Hook OnClicks so we start selecting stuff on left-click
	self:Hook("CalendarViewEventInviteListButton_OnClick", self.OnClick, true)
	self:Hook("CalendarCreateEventInviteListButton_OnClick", self.OnClick, true)

	core.RegisterCallback(self, "OpenEvent", function(event, frame)
		currentInviteList = frame
		lastSelectedIndex, lastSelectedName = 1, nil
		underline.shouldShow = nil
		self:UnselectAll()
	end)

	core.RegisterCallback(self, "Post_Update", "UpdateInviteListButtons")

	core.RegisterCallback(self, "Post_Sort", "RebuildSelectedIndices")

	self:RegisterEvent("CALENDAR_UPDATE_INVITE_LIST", function()
		listIsDirty = true
	end)

	highlightTexture = CalendarViewEventInviteList.scrollFrame.buttons[1]:GetHighlightTexture():GetTexture()
	for _, button in ipairs(CalendarViewEventInviteList.scrollFrame.buttons) do
		self:HookScript(button, "OnEnter", self.OnEnter)
		self:HookScript(button, "OnLeave", self.OnLeave)
	end
	self:Hook("CalendarEventInviteListButton_OnEnter", self.OnEnter, true) -- HookScripting OnEnter doesn't seem to work, but this does /shrug
	for _, button in ipairs(CalendarCreateEventInviteList.scrollFrame.buttons) do
		self:HookScript(button, "OnLeave", self.OnLeave)
	end
	self:HookScript(CalendarViewEventInviteListScrollFrame, "OnEnter", self.OnEnter)
	self:HookScript(CalendarViewEventInviteListScrollFrame, "OnLeave", self.OnLeave)
	self:HookScript(CalendarCreateEventInviteListScrollFrame, "OnEnter", self.OnEnter)
	self:HookScript(CalendarCreateEventInviteListScrollFrame, "OnLeave", self.OnLeave)

	-- Hook the right-click menu functions to make them apply to all of the selected players
	self:RawHook("CalendarInviteContextMenu_RemoveInvite", true)
	self:RawHook("CalendarInviteContextMenu_SetModerator", true)
	self:RawHook("CalendarInviteContextMenu_ClearModerator", true)
	self:RawHook("CalendarInviteContextMenu_InviteToGroup", true)
	self:RawHook("CalendarInviteStatusContextMenu_SetStatusOption", true)
	-- Override the default context-menu highlight-killer, we don't want that
	self:RawHook("CalendarInviteContextMenu_UnlockHighlights", function() end, true)

	-- Prepare the underline-frame
	underline = CreateFrame("frame")
	underline:Hide()
	underline:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"})
	underline:SetBackdropColor(1, 1, 1)
	underline:SetHeight(2)
	function underline:Reposition()
		local visible
		if self.shouldShow then
			for i, button in ipairs(currentInviteList.buttons) do
				if button.inviteIndex == lastSelectedIndex then
					visible = true
					self:SetParent(button)
					self:SetPoint("TOPLEFT", button, "BOTTOMLEFT", 2, 2)
					self:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT", -2, 2)
					self:Show()
					return
				end
			end
		end
		self:Hide()
	end
end

 --[[ -------- Hooks -------- ]]--

local function GetRealIndex(index)
	return (sortersModule and #sortersModule.reSortedList > 0 and sortersModule.reSortedList[index]) or index
end
 
function module:CalendarInviteContextMenu_RemoveInvite()
	for index in pairs(selectedByIndex) do
		CalendarEventRemoveInvite(GetRealIndex(index))
	end
end

function module:CalendarInviteContextMenu_SetModerator()
	for index in pairs(selectedByIndex) do
		CalendarEventSetModerator(GetRealIndex(index))
	end
end

function module:CalendarInviteContextMenu_ClearModerator()
	for index in pairs(selectedByIndex) do
		CalendarEventClearModerator(GetRealIndex(index))
	end
end

function module:CalendarInviteContextMenu_InviteToGroup()
	for k in pairs(selectedByName) do
		InviteUnit(k)
	end
end

function module:CalendarInviteStatusContextMenu_SetStatusOption(menu)
	for index in pairs(selectedByIndex) do
		CalendarEventSetStatus(GetRealIndex(index), menu.value)
	end
	CalendarContextMenu_Hide(CalendarCreateEventInviteContextMenu_Initialize)
end